/*
 *      inarray.c -- find if the element occurs in a bash array
 *      This file is part of zenpakt
 *
 *      Copyright 2011 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <config.h>

#if defined (HAVE_UNISTD_H)
#  include <unistd.h>
#endif
#include "bashansi.h"
#include <stdio.h>
#include <errno.h>

#include "builtins.h"
#include "shell.h"
#include "bashgetopt.h"
#include "strmatch.h"

int match_string (char *pattern, char *string, int pmatch, int flags) {
  if (pmatch)
    return (strmatch(pattern, string,flags) != FNM_NOMATCH)? 1 : 0;
  else
    return (strcmp(pattern, string) == 0)? 1: 0;
}

static int inarray (WORD_LIST *l, char *pattern, int pmatch, int flags) {
  int i=0;
  for (; l ; l=l->next, i++) {
#if 0
    printf ("i=%d ; word=%s; tofind=%s ;\n", i,l->word->word, pattern);
#endif
    if (match_string(pattern, l->word->word, pmatch, flags))
      return i;
  }
  return -1;
}

static int minarray(WORD_LIST *l,int d, int c, char *pattern, int pmatch, int flags) {
  int i=0, j=0;
  for (; l ; l=l->next, i++) {
    if ( i != c - 1 + (d*j) )
      continue;
#if 0
    printf ("i=%d ; j=%d ; word=%s; tofind=%s ;\n", i,j,l->word->word,pattern);
#endif
    if (match_string(pattern, l->word->word, pmatch, flags))
      return i;
    j++;
  }
  return -1;
}

/*Just iterate through the list to find if the word is same as the pattern/name*/
inarray_builtin (WORD_LIST *list) {
  char *index_varname=NULL, *tofind=NULL, *cindex=NULL;
  unsigned int opt, dimension=1, column=1, patternmatch=0, patternflags=0, indexlen;
  int index=-1;
  intmax_t ldimension, lcolumn;
  WORD_LIST *l;

  reset_internal_getopt ();
  while ((opt = internal_getopt (list, "i:m:c:PCE")) != -1) {
    switch (opt) {
      case 'i':
        if (legal_identifier (list_optarg) || valid_array_reference (list_optarg))
          index_varname=list_optarg;
        else {
          sh_invalidid (list_optarg);
          return EXECUTION_FAILURE;
        }
        break;
      case 'm':
        if (all_digits (list_optarg) && legal_number(list_optarg, &ldimension) && ldimension == (int)ldimension)
          dimension=ldimension;
        else
          return EXECUTION_FAILURE;
        break;
      case 'c':
        if (all_digits (list_optarg) && legal_number(list_optarg, &lcolumn) && lcolumn == (int)lcolumn)
          column=lcolumn;
        else
          return EXECUTION_FAILURE;
        break;
      case 'P':
        patternmatch=1;
        break;
      case 'C':
        patternflags |= FNM_CASEFOLD;
        break;
      case 'E':
        patternflags |= FNM_EXTMATCH;
        break;
      default:
        builtin_usage ();
        return (EX_USAGE);
    }
  }
  list = loptend;

  if (list == NULL)
    return EXECUTION_FAILURE;

  tofind = list->word->word;

  l=list->next;
  if (l == NULL)
    return EXECUTION_FAILURE;

  if (column > dimension || dimension < 1 || column < 1)
    return EXECUTION_FAILURE;

  if (dimension == 1)
    index=inarray(l, tofind, patternmatch, patternflags);
  else
    index=minarray(l, dimension, column, tofind, patternmatch, patternflags);

  if (index == -1)
    return EXECUTION_FAILURE;

  if (index_varname == NULL)
    return EXECUTION_SUCCESS;


  if (valid_array_reference (index_varname) == 0)
    bind_var_to_int (index_varname, index);
  else {
    indexlen=(sizeof(int) * CHAR_BIT + 2) / 3 + 1 + 1;
    cindex=xmalloc(indexlen);
    if ( snprintf(cindex, indexlen, "%d", index) > indexlen)
      return EXECUTION_FAILURE;
    assign_array_element (index_varname, cindex, 0);
    free(cindex);
  }

  stupidly_hack_special_variables (index_varname);

  return EXECUTION_SUCCESS;
}

char *inarray_doc[] = {
  "inarray [-PCE] [-m dimension] [-c col] [-i var] string list",
  ""
  "inarray looks through the arguments given as LIST and if an argument matches",
  "STRING, it returns 0 or else false.",
  "",
  "OPTIONS:",
  "-P  treat string as a pattern",
  "-C  do a case insensitive match, only if pattern matching is enabled",
  "-E  use extended matching",
  "-m Dimension",
  "    treat the list as false multi dimensional array of dimension DIMENSION",
  "-c column",
  "    look only in column COLUMN of the array of dimension DIMENSION",
  "    The column and dimensions start from 1",
  "-i var",
  "    Store the index of first match found in VAR",
  (char *)NULL
};

struct builtin inarray_struct = {
  "inarray",      /* builtin name */
  inarray_builtin,    /* function implementing the builtin */
  BUILTIN_ENABLED,    /* initial flags for builtin */
  inarray_doc,      /* array of long documentation strings. */
  "inarray [-PCE] [-m dimension] [-c col] [-I var] string list",     /* usage synopsis; becomes short_doc */
  0       /* reserved for internal use */
};
