/*
 *      metaget.c
 *      This file is part of zenpakt
 *
 *      Copyright 2011 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
 *
 *      Zenpakt is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */



/* meta-get [-n] filename
 * get file list from the meta databases for given file on the command
 * line.If -n is specified produce a null delimited output.
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>


#if defined (HAVE_UNISTD_H)
#  include <unistd.h>
#endif

#include "builtins.h"
#include "shell.h"
#include "bashgetopt.h"
#include "chop.h"
#include "escape.h"

/*the two modes of obtaining file list*/
enum {
  METAGET_FILELIST_ESC,
  METAGET_FILELIST_UNESC
};

static void print_unescaped_file_name(char *line, int len);
static int printfilelist(int argc, char** argv, int mode);

static void print_unescaped_file_name(char *line, int len) {
  char eline[len];
  do_chop(line);
  unescape(line, eline);
  printf("%s%c", eline, '\0');
}

static int printfilelist(int c, char **list, int mode) {

  FILE * file;
  char * line = NULL;
  size_t len = 0;
  ssize_t read;
  int start=0, i;

  for (i=0 ; i < c ; i++, start=0) {
    if (!list[i])
      continue;
    if ((file = fopen (list[i], "r")) == NULL) {
      char *s=strerror(errno);
      FILE *stderr=fdopen(dup(2),"w");
      fprintf(stderr,"error opening file:%s:%s\n",list[i],s);
      fclose(stderr);
      return 1;
    }

    while ( (read = getline(&line, &len, file)) != -1 ) {
      if (start == 0 && strcmp(line, "FILE LIST:\n") == 0)
	start=1;
      else if(read == 1 && line[0] == '\n' && start) {
	start=0;
	fflush(stdout);
	break;
      } else if (start) {
	switch(mode) {
	  case METAGET_FILELIST_UNESC:
            print_unescaped_file_name(line, read+1);
	    break;
	  case METAGET_FILELIST_ESC:
	    do_chop(line);
	    printf("%s\n", line);
	  default:
	    break;
	}
      }
    }

    fclose(file);
  }

if (line)
  free(line);

return 0;
}

int metaget_builtin (WORD_LIST *list) {
  int c, opt, mode=METAGET_FILELIST_ESC;
  char **v;

  reset_internal_getopt ();
  while ((opt = internal_getopt (list, "fF")) != -1) {
    switch (opt) {
      case 'f':
        mode=METAGET_FILELIST_ESC;
	break;
      case 'F':
        mode=METAGET_FILELIST_UNESC;
	break;
      default:
	builtin_usage();
	return(EX_USAGE);
    }
  }

  list = loptend;

  v=make_builtin_argv(list,&c);

  if (c == 1) {
    builtin_usage();
    return EX_USAGE;
  }

  if (printfilelist(c,v+1,mode))
    return EXECUTION_FAILURE;

  return EXECUTION_SUCCESS;
}

char *metaget_doc[] = {
  "metaget [-fF] filename...",
  "",
  "get newline delimited file lists from given meta database",
  "-F specifies that print a null delimited unescaped file list.",
  "-f specifies that print a newline delimited file list without",
  "unescaping backslashed quoted characters.It is recommended you use",
  "-f as default behaviour may change in future",
  (char *)NULL
};

struct builtin metaget_struct = {
  "metaget",		/* builtin name */
  metaget_builtin,		/* function implementing the builtin */
  BUILTIN_ENABLED,	/* initial flags for builtin */
  metaget_doc,		/* array of long documentation strings. */
  "metaget [-fF] filename...",		/* usage synopsis; becomes short_doc */
  0			/* reserved for internal use */
};
