#       getargs.bash
#       This file is part of zenpakt

#       Copyright 2011 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC
=========
 getargs
=========

-----------------------------
 parse positional parameters
-----------------------------

SYNOPSIS
--------
 **getargs** [ **-s** ] [ **-l** ] [ **-E** *EOO* ] *callback* *arguments*...

DESCRIPTION
-----------

parse the positional parameters according to values specified in array named as
getargs. The callback is evaluated for each argument found and is given two extra
arguments the anchor and the option and a third argument in case the option
accepts an argument.If callback is null, true is called.If callback returns
non-zero, getargs returns non-zero immediately. Special long arguments version
and help are recognised internally and do not need to be passed in getargs,
they are matched before any other long option matches and including them
in getargs would be pointless.The callback is called for these special
options and it is upto callback to either use them, ignore them or return
non zero stating the option is illegal.

getargs
~~~~~~~

  getargs must be a pseudo 4-dimensional array with the following
  columns for each row (in that order):

  *  *short*
  *  *long*
  *  *argument*
  *  *repeat*

  *short* is the first column which defines a short option.A short option must be
  single character long, since only single characters are used to match short options
  Thus making a short option either NULL or more than one character ensures that
  it would not match any short option, effectively stating that there is no short option
  for this case.

  *long* defines a long option, it can be any length, if NULL means there is no long
  option.

  if *argument* is :, it means that the option requires an argument, anything else means
  it doesnot require an argument.

  *repeat* represents how many times the argument can be repeated, if 0 means that option
  should not be present at all.If negative number then, it means it can be repeated
  infinite number of times.

OPTIONS
-------

-s anchors
  an | separated list of short anchors, you must quote any globbing characters twice
  (default is -)
-l anchors
  an | separated list of long anchors, you must quote any globbing characters twice
  (default is --)
-E EOP
  the | separated list for end of options, by default it is -|--

Return Status
-------------

*  0 success
*  1 failure
$ENDDOC

$$

getargs() {
#==========
#functions
#==========
  _getargs_callback() { eval "${_getargs[callback]}" "$@" ;}
  #check the repeat value
  #if it is a negative number then , repeat is infinite, no need to remove the
  #entry from getargs_short and getargs_long
  #if it is 0 then , limit is exceeded (it could have been set initially in getargs)
  #if it is anything else, reduce the repeat values
  #_getargs_repeat index option argument
  _getargs_repeat() {
    ((${getargs[$1]} == 0 )) && return 1
    #reduce the repeat each time, for non negative repeat
    ((${getargs[$1]} > 0 ? getargs[$1]-- : 1))
  }

  #parse one or more concatenated short options
  #_getargs_set_args_short anchor option_without_anchor [next_options]
  _getargs_short() {
    #iterate over the list, each character is/should be a short option
    #if a short option requires an argument, remaining is the argument
    for ((_getargs[i]=0 ; _getargs[i] < ${#2} ; _getargs[i]++)) ; do

      #check if the option is in the expected option list
      inarray -i _getargs[index] -m 4 -c 1 "${2:_getargs[i]:1}" "${getargs[@]}" || return 1
      if [[ ${getargs[_getargs[index]+2]} = : ]] ; then
        if (( _getargs[i] + 1 == ${#2} )) ; then
          if [[ $3 ]] ; then
            _getargs_repeat "$((_getargs[index]+3))" "${2:_getargs[i]:1}" "$3" || return 1
            #call the call back now
            _getargs_callback "$1" "${2:_getargs[i]:1}" "$3"  || return 1
            _getargs[shift]=2
            break
          else
            error_print "no argument specified for ${2:_getargs[i]:1}"
            return 1
          fi

        #use the remaining string as argument
        else
          _getargs_repeat "$((_getargs[index]+3))" "${2:_getargs[i]:1}" "${2:_getargs[i]+1}" ||
          return 1
          _getargs_callback "$2" "${2:_getargs[i]:1}" "${2: ++_getargs[i]}" ||
          return 1
          _getargs[shift]=1
          break
        fi

      #if we dont need an option
      else
        _getargs_repeat "$((_getargs[index]+3))" "${2:_getargs[i]:1}" || return 1
        _getargs_callback "$1" "${2:_getargs[i]:1}" || return 1
        _getargs[shift]=1
      fi

    done
  }

  #set the long option
  #_getargs_set_args_long anchor option_without_anchor next_argument
  _getargs_long() {
    if [[ $2 = version || $2 = help ]] ; then
      _getargs_callback "$1" "$2" || return 1
      _getargs[shift]=1
      return 0
    fi
    inarray -i _getargs[index] -m 4 -c 2 "$2" "${getargs[@]}" || return 1
    if [[ ${getargs[_getargs[index]+1]} = : ]] ; then
      [[ $3 ]] || return 1
      _getargs_repeat "$((_getargs[index]+2))" "$2" "$3" || return 1
      _getargs_callback "$1" "$2" "$3" || return 1
      _getargs[shift]=2

    else
      _getargs_repeat "$((_getargs[index]+2))" "$2" || return 1
      _getargs_callback "$1" "$2" || return 1
      _getargs[shift]=1
    fi
  }

  #################
  #local variables
  #################
  declare -A _getargs
  _getargs=(
    #list of | separated short anchors; default anchor is -
    [anchor_s]=-
    #list of | separated long anchors; default anchor is --
    [anchor_l]=--
    #the end of options
    [EOP]='-|--'
    # callback
    [callback]=
    #returned by internal functions to specify how much to shift
    [shift]=
    #used internally by internal functions to get the index of option via inarray
    [index]=
    #internal variable used in for loops and other things
    [i]=
    #original number of arguments
    [argc]=
  )

  ################
  #OPTIONS
  ################
  #<irony> parse our own options using getopts
  local _getargs_opt OPTIND=1 OPTARG OPTERR
  while getopts :s:l:E: _getargs_opt; do
    case $_getargs_opt in
      s)
        _getargs[anchor_s]=$OPTARG
        ;;

      l)
        _getargs[anchor_l]=$OPTARG
        ;;

      E)
        _getargs[EOO]=$OPTARG
        ;;

      \?|:)
        die "invalid usage"
        return 1
        ;;

     esac
  done
  shift $((OPTIND-1))

  #==============
  # THE CALLBACK
  #==============
  _getargs[callback]=${1:-true}
  shift

  _getargs[argc]=$#

  ######################
  #CHECK ARRAY STRUCTRES
  ######################
  #check if getargs is not empty and is a 4 dimensional array(in multiples of 4)
  ((${#getargs[@]} ? (${#getargs[@]}%4 == 0) : 0)) || {
    die "array getargs not defined"
    return 1
  }

  ######################
  #EXTERNAL VARIABLES
  ######################
  #clear up the variables now
  GETARGS_IND= || return 1

  #=================
  # PARSE ARGUMENTS
  #=================
  _getargs_opt=
  while (($#)) ; do

    #any of the anchors specifying the start of an option alone signals end of
    #options, so does anything not starting with these anchors
    [[ $1 = @(${_getargs[EOO]}) ]] &&
    shift && break

    #if the option starts with a short anchor and not with a long anchor
    #we may/should have a short option
    if [[ $1 = @(${_getargs[anchor_s]})* && $1 != @(${_getargs[anchor_l]})* ]];then

      #remove the longest possible matching short anchor
      _getargs_opt=${1##@(${_getargs[anchor_s]})}
      #If only anchor was given and it was not EOO its an error
      [[ $_getargs_opt ]] || return 1
      _getargs_short "${1%$_getargs_opt}" "$_getargs_opt" "${@:2}" || return 1
      shift "${_getargs[shift]}"

    #if the option starts with a long option option and not a short one
    elif [[ $1 = @(${_getargs[anchor_l]})* && $1 != @(${_getargs[anchor_s]})* ]] ; then
      _getargs_opt=${1##@(${_getargs[anchor_l]})}
      #If only anchor was given and it was not EOO its an error
      [[ $_getargs_opt ]] || return 1
      _getargs_long "${1%$_getargs_opt}" "$_getargs_opt" "${@:2}" || return 1
      shift "${_getargs[shift]}"

    #if option matches both long and short anchors
    elif [[ $1 = @(${_getargs[anchor_l]})* && $1 = @(${_getargs[anchor_s]})* ]] ; then

      #choose the longer anchor one
      _getargs_opt=${1##@(${_getargs[anchor_l]})}
      _getargs_opt[1]=${1%$_getargs_opt}
      _getargs_opt=${1##@(${_getargs[anchor_s]})}
      _getargs_opt=${1%$_getargs_opt}

      if ((${#_getargs_opt[0]} > ${#_getargs_opt[1]})) ; then
        #we have short options
        #If only anchor was given and it was not EOO its an error
        [[ $1 = $_getargs_opt ]] && return 1
        _getargs_short "$_getargs_opt" "${1#$_getargs_opt}" "${@:2}" || return 1
        shift "${_getargs[shift]}"
      else
        #If only anchor was given and it was not EOO its an error
        [[ $1 = ${_getargs_opt[1]} ]] && return 1
        _getargs_long "${_getargs_opt[1]}" "${1#${_getargs_opt[1]}}" "${@:2}" || return 1
        shift "${_getargs[shift]}"
      fi

    else
      #it is end of options
      break
    fi
  done
  ################
  #END PARSING
  ################

  #set the index and return
  ((GETARGS_IND = $# ?  _getargs[argc] - $# + 1 : _getargs[argc] + 1))
}
