#       mktemp.bash
#       This file is part of zenpakt

#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC
========
 mktemp
========

SYNOPSIS
--------

mktemp var [-p dir] [-d] [-t] [template]

mktemp_bash var [-p dir] [-d] [-t] [template]

mktemp_init_tmpdir parent_dir

mktemp_rm_tmpdir

DESCRIPTION
-----------

mktemp() creates a temporary file using GNU mktemp if available or else using
bash $RANDOM sequences by calling mktemp_bash.The template may be any filename
with six X appended to it.If no template is specified a default of tmp.XXXXXX is
used and -t is implied.

By default it creates directories and files in current directory if the given
template doesnot contain a directory path.If -t is used or if the template is not
given -t is implied, the directory for creating temporary files is chosen as
follows:

*  if _mktemp_TMPDIR variable is set, directory contained there in is used
*  else if -p was given the specified directory is used.
*  else /tmp is used.

With -t and -p, template used must not contain a directory path.

As a general rule, we always create a temporary directory and create temporary
files within and clean the directories up on exit.

If a command mktemp is found in PATH, GNU mktemp is assumed.


OPTIONS
-------
  -p  the parent directory, implies -t
  -d  create a directory instead
  -t  create file under a temporary directory

Note: the -q option is used by default with mktemp utility

RETURN STATUS
-------------

*  0  success
*  >0  failure

ENVIRONMENT
-----------

*  _mktemp_TMPDIR

   The value set by this variable is used as value for TMPDIR for mktemp call
   we dont set TMPDIR ourseleves but use this variable.

   mktemp_bash() refers to this variable directly and mktemp calls the command
   mktemp with value of TMPDIR set to the value of thei variable in the command
   execution environment. Therefore, if you are using external mktemp do not
   make TMPDIR readonly.

   This variable can be initailized by calling mktemp_init_tmpdir with a parent_dir
   argument and can be removed by calling mktemp_rm_tmpdir() in the end. Do not
   use this variable directly.

*  ZENPAKT_USE_MKTEMP

   If value is zero, mktemp_bash is used instead.

$ENDDOC


#############
#NEW mktemp
#############

$$

#The value of this variable will come from ./configure in future
ZENPAKT_USE_MKTEMP=0

#This is an internal variable initailized by and set to temporary directory
#by mktemp_init_tmpdir.The value of this variable is used as value of TMPDIR
#for a call to mktemp or mktemp_bash with -t option or without any template.
_mktemp_TMPDIR=

mktemp() {
  local _mktemp_file
  if ((ZENPAKT_USE_MKTEMP)) && chk_cmd mktemp ;then
    _mktemp_file=$(TMPDIR=$_mktemp_TMPDIR command mktemp -q "${@:2}") ||
    return 1
    printf -v "$1" '%s' "$_mktemp_file"

  else
    mktemp_bash "$@"
  fi
}

mktemp_bash() {
  local OPTIND=0 OPTARG OPTERR _mktemp_opt \
    _mktemp_pdir _mktemp_dir=0 _mktemp_i _mktemp_var=$1 _mktemp_use_tmpdir=0 \
    _mktemp_file _mktemp_file_basename

  shift
  while getopts :p:d _mktemp_opt ; do
    case $_mktemp_opt in
      p)
        [[ -d $OPTARG ]] || return 1
	_mktemp_pdir=$OPTARG
	_mktemp_use_tmpdir=1
	;;
      d)
	_mktemp_dir=1
	;;
      t)
        _mktemp_use_tmpdir=1
	;;
    esac
  done

  shift $((OPTIND-1))

  if [[ $1 ]];then
    _mktemp_file=${1%XXXXXX}
  else
    _mktemp_file=$RANDOM
    _mktemp_use_tmpdir=1
  fi

  if [[ $1 = */* ]] ; then
    _mktemp_file_basename=${_mktemp_file##*/}
    _mktemp_file=${_mktemp_file%/*}/$_mktemp_file_basename$RANDOM$$$RANDOM
  else
    _mktemp_file=$_mktemp_file$RANDOM$$$RANDOM
  fi

  if ((_mktemp_use_tmpdir)) ; then
    [[ $1 = */* ]] && return 1
    if [[ -d $_mktemp_TMPDIR ]] ; then
      _mktemp_pdir=$_mktemp_TMPDIR
    elif [[ ! $_mktemp_pdir && -d /tmp ]] ; then
      #If temporary directory is variable is non null but not present something is wrong
      assert '[[ ! $_mktemp_TMPDIR ]]'
      _mktemp_pdir=/tmp
    else
      return 1
    fi

    _mktemp_file=$_mktemp_pdir/$_mktemp_file
  fi

  if ((_mktemp_dir)) ; then
    mkdir -m 700 "$_mktemp_file" || return 1
  else
    rm -rf "$_mktemp_file" || return 1
    (set -C ; > "$_mktemp_file") || return 1
  fi

#Uncomment to check if the temporary files are created properly
#  debug_msg "mktemp: $_mktemp_var=$_mktemp_file"
  printf -v "$_mktemp_var" '%s' "$_mktemp_file"
}

mktemp_init_tmpdir() {
  mktemp _mktemp_TMPDIR -d "$1/${RANDOM}XXXXXX" || return 1
  readonly _mktemp_TMPDIR
}

mktemp_rm_tmpdir() {
  rm -rf "$_mktemp_TMPDIR"
}
