#       pkg_parsepkgname.bash
#
#       This file is part of zenpakt
#
#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC
================
 pkg_parse_name
================

SYNOPSIS
--------
**pkg_parse_name** *package* *abs_name* *basename* *compression* *name* [ *version*
*arch* *release* *build* *extension* ]

DESCRIPTION
-----------

pkg_parse_name parses *package*\'s name to get the extension and other info.
Remaining arguments are treated as variables to store the corresponding
information:

*  *abs_name*  the absolute path to the package
*  *basename*

   the name of the package without the directory and extension but still
   with the version, build, architecture and release

*  *compression*  the compression utility for the package
*  *name*  the basename of the package stripped of version , build, architecture and release.
*  *version*  the version number for the package
*  *arch*  the architecture for the package
*  *release*  the Zenwalk release for the package
*  *build*  the build numer

RETURN STATUS
-------------
*  0  success
*  >0 It returns 1 if extension is not valid

Return status is zero even if no version, architecture, build, release are found.
The value of these variables is unchanged if no value is found.As such, if they
were not set or null before a call to this function they still will be after a
call.

$ENDDOC

$$

  #nomenclature of package: package-basename-version-arch-zenwalkrelease.build.format
  #old style package names: package-basename.format
  #                         package-basename-version-arch-build.format ( with no - in the package-basename )
  # New style package name: package-basename-version-arch-build.format ( with one or more - in the package-basename)
pkg_parse_name() {
  [[ $1 != *.t[gblx]z ]] && return 1

  #store the fullpath to package in $2
  fullpath "$1" "$2"

  local _pkg_parse_name_ext _pkg_parse_name_base

  _pkg_parse_name_ext=${1##*.}
  _pkg_parse_name_base=${1##*/} _pkg_parse_name_base=${_pkg_parse_name_base%.t[gblx]z}

  #store the basename in $3
  printf -v "$3" '%s' "$_pkg_parse_name_base"

  #store compression in $4
  case $_pkg_parse_name_ext in
    tgz)
      printf -v "$4" gzip
      ;;
    tbz)
      printf -v "$4" bzip2
      ;;
    tlz)
      printf -v "$4" lzma
      ;;
    txz)
      printf -v "$4" xz
      ;;
  esac

  #store package name in $5
  #It is same as basename in case there are less than 3 -'s, old-style name
  printf -v "$5" '%s' "${_pkg_parse_name_base%-*-*-*}"

  #if additional/optional arguments are given
  if (($# > 5)) ; then

    # a package must have at least three -'s separated fields to qualify for
    #following information
    if [[ $_pkg_parse_name_base = *-*-*-* ]] ; then

      local _pkg_parse_name_data=() _pkg_parse_name_build

      IFS=- read -a _pkg_parse_name_data <<<"$_pkg_parse_name_base"

      #store version in $6 and architecture in 7
      printf -v "$6" '%s' "${_pkg_parse_name_data[@]: -3:1}"
      printf -v "$7" '%s' "${_pkg_parse_name_data[@]: -2:1}"

      #Store the release number in $8
      _pkg_parse_name_build=${_pkg_parse_name_data[@]: -1}
      printf -v "$8" '%s' "${_pkg_parse_name_build%.*}"

      #Store the build number in $9
      _pkg_parse_name_build=${_pkg_parse_name_build#*.}
      printf -v "$9" '%s' "$_pkg_parse_name_build"

    fi

    printf -v "${10}" '%s' "$_pkg_parse_name_ext"
  fi
}
