$INCLUDE ./config.bash
$$

#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt
#
#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

#========
# SOURCE
#========
#source required .functions files
. "$ZENPAKT_INCLUDE_DIR/"common installpkg || {
  printf '%s\n' "failed to source functions" >&2
  exit 1
}

#source configfile here using sourceconf() in common.functions,for next zenpakt
#version

#=========================
# CONFIGURATION VARIABLES
#=========================

ZENPAKT_PROG_VERSSTR="installpkg (zenpakt)"

MD5SUM=0         # Do not store md5sums by default ; following installpkg behaviour

PARANOID=1       # backup files before installing ; recommended but little slow

# default ROOT is / ;specify another as an env var or use -r|--root
if [[ -d $ROOT ]] ; then
  fullpath "$ROOT" ROOT
else
  ROOT=/
fi

MODE=install     # default MODE ; <do not change>

# verbose is set to 1 ; this means you will see all messages and errors
VERBOSE=1

# install a package even if it overwrites a previous package index
INSTALLPKG_OVERWRITE=1

ASK=0            # non-interactive mode by default
RAW=0            # no-spin or raw mode is off by default

#============
#  MAIN LOOP
#============

# main loop function that takes package names as arguments and attempts to install or warn according to the $MODE set
installpkg_main() {

  local package REPLY compression basename name compressed uncompressed \
    full_desc desc md5sum installedpkglist=()

  for package do
    #initial checks that prevent further processing in case a necessary condition
    #for a package is not met with; this introduces a race condition
    #there should always be a double check for possible failures during actual
    #installation

    spin_msg "verifying $package..."

    #check wether the package is a file or url
    pkg_is_file_or_url "$package" REPLY ||
    error_continue "$ERR_NOTEXIST" "$package"

    [[ $REPLY != file ]] && error_continue "$ERR_NOTEXIST"  "$package"

    #if [[ $pkg_pkgtype = url ]] ; then
      #error 30 "$package"
      #if ((ASK)) ;then
  #Select -n1 -Cp"download url package '$package'(y/n).." y n
  #[[ $REPLY != [yY] ]] && error 32 "$package"
      #fi

      #wfetch "$package" || error 33 "$package"
      #package=$pkg_urlpkg
    #fi

    #get various data obtained from name of package
    #ensure full path for $package, as we will be cd'ing later
    pkg_parse_name "$package" package basename compression name ||
    error_continue "$ERR_INVAL_EXT" "$package"

    spin_msg_success

    #check if we are overwriting package index
    [[ $INSTALLPKG_OVERWRITE = 0 && -f $ADM_DIR/packages/$basename ]] &&
    error_continue "$ERR_INSTALLED" "$package"

    spin_msg "checking for $compression..."

    # if external compression utility doesn't exist
    chk_cmd "$compression" || error_continue "$ERR_NO_COMPRESSUTIL" "$package"

    spin_msg_success

    case "$MODE" in

      warn)
        #error between 61 and 62 are for warn mode
        installpkg_warn "$package" "$compression" || error_continue $((60+$?)) "$1"
        ;;

      install|ask|raw|quiet)
        # get description for the package , error may occur over here in
        #determining uncompressed size or extracting install but is not very
        #significant at this step to stop the installation of the package

        spin_msg "looking for package description..."
        pkg_desc "$package" "$compression" "$basename" "$name" compressed \
          uncompressed full_desc desc md5sum
        spin_msg_success

        print_v "" "${TXT_bold}PACKAGE: ${TXT_reset}$basename" \
          "${TXT_bold}PACKAGE DESCRIPTION:${TXT_reset}"
        echon_v "$desc"
        print_v "${TXT_bold}SIZE:${TXT_reset}" \
          "Compressed: $compressed" \
          "Uncompressed: $uncompressed" ""

        if ((ASK)) ; then
          yesno "install $basename at '$ROOT'" || continue
        fi

        if ((PARANOID)) ; then
          spin_msg "backing up files..."

          installpkg_bkup "$package" "$compression" "$basename" ||
          error_continue $? "$package"

          spin_msg_success
        fi

        spin_msg "installing $basename..."

        installpkg_state_start "$basename"

        if installpkg_install "$package" "$compression" "$basename" \
          "$compressed" "$uncompressed" full_desc "$md5sum" ; then
          installedpkglist+=("$basename")
          #remove any partial database entry
          rm -f "$ADM_DIR/partial_packages/$basename"
          spin_msg_success

        else
          status=$?
          installpkg_move_db_partial "$basename"
          ((PARANOID)) && installpkg_bkup_restore
          installpkg_state_end
          error_continue $status "$package"
        fi

        spin_msg "configuring $basename..."

        installpkg_configure "$package" "$compression" "$basename" || {
          status=$?
          ((PARANOID)) && installpkg_bkup_rm
          installpkg_state_end
          error_continue $status "$package"
        }

        spin_msg_success

        #Remove the backup now; we wait till now to remove the backup so
        #we can restore if we receive INT or TERM
        ((PARANOID)) && installpkg_bkup_rm

        installpkg_state_end
        ;;
    esac
  done
  #summary of error messages is not displayed for warn mode
  #in future error codes will be displayed for warn mode also instead of displaying it there and then
  [[ $MODE = warn ]] && return 0

  #((ASK)) && installpkg_clear $# $#

  ((${#installedpkglist[@]})) && print_v "installed the following package(s):" \
    "${installedpkglist[@]}"

  error_summary

   if ((ASK)) ; then
    read -n 1 -p "${TXT_bold}press any key to continue${TXT_reset}"
  fi
}

#exit on SIGINT or SIGTERM
#Using error_print terminates an active spin too
trap 'error_print "received SIGINT, exiting..." ; exit' INT
trap 'error_print "received SIGTERM, exiting..." ; exit' TERM

#check wether the user is root or not , this is just a check for code clarity and
#in future can be overridden by option such as --notroot
((UID)) && error_exit "$ERR_NOTROOT"

#Set proper umask
umask 022

#parse arguments and if it fails we exit
installpkg_parseargs "$@" || error_exit "$ERR_OPT"

#check tar exists and if so which version
tar || error_exit "$ERR_NO_TAR"

#Do some clean up on exit
trap '((BASHPID == $$)) && installpkg_cleanup' EXIT

#create a lock to prevent parallel run
lock || error_exit "$ERR_LOCKED"

#initialize meta database if it doesnt already exist
meta_init_db || error_exit "$ERR_DBINIT"
meta_init_tmpdir || error_exit "$ERR_CREAT_TMP"

#Get some colors and text attributes needed in case both
#RAW is zero and VERBOSE is one
if ((RAW == 0 && VERBOSE == 1)) ; then
  col_fg green red cyan
  txt_attrib bold reset
fi

if ((ASK)) ; then
  ask_init || error_exit $?
fi

#main loop function
installpkg_main "${@:GETARGS_IND}"

#exit with the last set exitstatus
error_exit
