$INCLUDE ./config.bash
$$

#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt

#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

#========
# SOURCE
#========
#source required .functions files
. "$ZENPAKT_INCLUDE_DIR"/common removepkg || {
  printf "%s\n" "failed to source function" >&2
  exit 1
}

#source configfile here using sourceconf() in common.functions, for next zenpakt
#version

#=========================
# CONFIGURATION VARIABLES
#=========================

ZENPAKT_PROG_VERSSTR="removepkg (zenpakt)"

# extra checks while removing packages
PARANOID=1

# default ROOT is / ;specify another as an env var or use -r|--root
if [[ -d $ROOT ]] ; then
  fullpath "$ROOT" ROOT
else
  ROOT=/
fi

# default MODE ; <do not change>
MODE=remove

# verbose is set to 1 ; this means all messages and errors output to screen
VERBOSE=1

# non interactive mode by default
ASK=0

# no-spin or raw mode is off by default
RAW=0

#quit if more than this number of packages are found to be matching the desired
#pkgname given in removeall mode
REMOVEPKG_MAXRM=10


#default behaviour like the original removepkg is to remove one among many
#matches.If set to 1, remove all packages if number of matches is less than
#REMOVEPKG_MAXRM.If set to 2, remove only packages if there is a unique match
#for specified name
REMOVEALL=0

#set to currently being processed package
#removepkg_currentpkg=

#============
#  MAIN LOOP
#============

#main loop function that takes package names as arguments and attempts to
#remove or warn according to the $MODE set sequence:
#   if interactive mode then clear screen and set the header:removepkg_clear()
#   get possible matching installed package names for given names as arguments
#   if more than one match is found,
#      if --unique was specified, give an error and continue to next argument
#      if --removeall was specified and number of matches is less than
#        REMOVEPKG_MAXRM as set by --max, then remove all the matches
#      if --ask was specified,ask the user to select from the list of matches
#      else just choose one among the sorted ones( like default pkgtools
#      removepkg )
#   for list of packages selected as above , warn if --warn was specified or
#   remove the packages
#   if MODE is not warn then print packages removed <in verbose modes>
#   if MODE is not warn then print errors for packages <in all other modes>

removepkg_main() {

  local package selected=() found=() toberemoved=()

  for package do

    #initial checks that prevent further processing in case a necessary
    #condition for a package is not specified; this introduces a race condition
    #there should always be a double check for possible failures during
    #removal

    spin_msg "searching for '$package'..."

    #check for existence of package and guess installed name
    found=()
    removepkg_find_pkg "$package" found || error_continue $? "$package"

    spin_msg_success

    #select among found packages
    selected=()
    removepkg_select_pkg selected "${found[@]}" || error_continue $? "$package"

    #selected may be empty in case of ask mode
    ((${#selected[@]})) || error_continue "$ERR_META_NOTSELECTED" "$package"

    #Add the packages to the list of packages to be removed
    toberemoved+=( "${selected[@]}" )

  done

  ((${#toberemoved[@]})) || {
    error_summary
    error_exit "$ERR_META_NOTSELECTED"
  }

  spin_msg "fetching file list..."

  local list
  mktemp list || error_exit "$ERROR_CREAT_TMP"
  removepkg_file_list_unique "${toberemoved[@]}" > "$list" || error_exit $?

  spin_msg_success

  case $MODE in
    warn)
      removepkg_warn_files < "$list"
      removepkg_warn_dirs <"$list"
      removepkg_warn_skipped
      ;;

    remove|ask|raw|quiet)

      #RAW is treated specially here because we output full list of files being
      #removed and generally do not suppress any error messages
      #As such usual spin_msg followed by spin_msg_success cannot be used.
      if [[ $MODE = raw ]]; then

        printf '%s\n' ">>>Removing files:"

        #First set the state to start in case PARANOID is non zero
        removepkg_state_start

        removepkg_remove_files < "$list" || {
          error $?
          error_summary
          error_exit
        }

        printf '%s\n' ">>>Moving Database and post-install scripts..."
        removepkg_move_dbentry "${toberemoved[@]}" || {
                error $?
          error_summary
          error_exit
        }

        #No backup is done for following steps and restore is not necessary
        #So we end the state now
        removepkg_state_end

        removepkg_remove_cats < "$list"

        printf '%s\n' "-->Removing empty directories..."

        removepkg_remove_dirs < "$list"

      else
        spin_msg "removing files..."

        #First set the state to start in case PARANOID is non zero
        removepkg_state_start

        removepkg_remove_files < "$list" >/dev/null 2>&1 || {
          error $?
          error_summary
          error_exit
        }

        spin_msg_success

        spin_msg "Moving Database and Post-install scripts..."

        removepkg_move_dbentry "${toberemoved[@]}" >/dev/null 2>&1 || {
          error $?
          error_summary
          error_exit
        }

        #No backup is done for following steps and restore is not necessary
        #So we end the state now
        removepkg_state_end

        removepkg_remove_cats < "$list"

        spin_msg_success

        spin_msg "removing empty directories..."

        removepkg_remove_dirs < "$list" >/dev/null 2>&1

        spin_msg_success

      fi

      if ((VERBOSE)) ; then
        print_v ">>>Skipped files and directories(found in other package(s)):" >&2
        removepkg_file_list_common >&2

        print_v ">>>Skipped unique directories(not empty):" >&2
        removepkg_remove_dirs_print_skipped >&2

        error_summary
      fi

      ((ASK)) && read -n 1 -p "${TXT_bold}press any key to continue${TXT_reset}"
      ;;
  esac

}

#exit on SIGINT or SIGTERM
#Using error_print terminates an active spin too
trap 'error_print "received SIGINT,exiting..." ; exit' INT
trap 'error_print "received SIGTERM,exiting..." ; exit' TERM

#check wether the user is root or not , this is just a check for code clarity
((UID)) && error_exit "$ERR_NOTROOT"

#Set proper umask
umask 022

#parse arguments and if it fails we exit
removepkg_parseargs "$@" || error_exit "$ERR_OPT"

#check tar exists and if so which version
tar || error_exit "$ERR_NO_TAR"

#enable_builtin unconvesc || error_exit "$ERR_BUILTIN_LOAD"
#enable_builtin doinstxlinks || error_exit "$ERR_BUILTIN_LOAD"

# check if package meta database in given root exists or not
[[ -d $ROOT/var/log/packages ]] || error_exit "$ERR_NO_DB"

#Do some clean up on exit
trap '((BASHPID == $$)) && removepkg_cleanup' EXIT

# check lock or create a lock to prevent parallel run
lock || error_exit "$ERR_LOCKED"

# initialize tmpdir, package database should already be there, at least
#partially
meta_init_db || error_exit "$ERR_DBINIT"
meta_init_tmpdir || error_exit "$ERR_CREAT_TMP"

#Get some colors and text attributes needed in case both
#RAW is zero and VERBOSE is one
if ((RAW == 0 && VERBOSE == 1)) ; then
  col_fg green red cyan
  txt_attrib bold reset
fi

if ((ASK)) ; then
  ask_init || error_exit $?
fi

#main loop function
removepkg_main "${@:GETARGS_IND}"

#exit with the last set exitstatus
error_exit
